@@ -2,9 +2,9 @@
 import base64
 import pickle
 import random
-
 from collections import deque
 from copy import copy
+from hashlib import md5
 try:
     from itertools import combinations
 except ImportError:
@@ -22,10 +22,19 @@ from django.db.models.fields.related import (ForeignKey, OneToOneField,
                                              ManyToManyField)
 from django.core.exceptions import SuspiciousOperation
 from django.conf import settings
-from django.utils.hashcompat import md5_constructor
 from django.utils.importlib import import_module
 from django.utils.simplejson import dumps
 
+try:
+    # Default value to backwards compatibility
+    qbe_admin_site = getattr(settings, "QBE_ADMIN_SITE", "admin.admin_site")
+    qbe_admin_site_splits = qbe_admin_site.rsplit(".", 1)
+    qbe_admin_module = qbe_admin_site_splits[0]
+    qbe_admin_object = qbe_admin_site_splits[1]
+    admin_site = getattr(import_module(qbe_admin_module), qbe_admin_object)
+except (AttributeError, ImportError):
+    from django.contrib.admin import site as admin_site
+
 try:
     from django.db.models.fields.generic import GenericRelation
 except ImportError:
@@ -93,7 +102,7 @@ def qbe_models(admin_site=None, only_admin_models=False, json=False):
             'model': field.rel.to.__name__,
             'field': field.rel.to._meta.pk.name,
         }
-        if hasattr(field.rel, 'through'):
+        if hasattr(field.rel, 'through') and field.rel.through is not None:
             name = field.rel.through.__module__.split(".")[-2]
             target.update({
                 'through': {
@@ -143,6 +152,8 @@ def qbe_models(admin_site=None, only_admin_models=False, json=False):
 
         if app_model._meta.many_to_many:
             for field in app_model._meta.many_to_many:
+                if not hasattr(field, 'primary_key'):
+                    continue
                 field_attributes = get_field_attributes(field)
                 model['fields'].update(field_attributes)
                 if isinstance(field, ManyToManyField):
@@ -399,8 +410,7 @@ def autocomplete_graph(admin_site, current_models, directed=False):
 def pickle_encode(session_dict):
     "Returns the given session dictionary pickled and encoded as a string."
     pickled = pickle.dumps(session_dict, pickle.HIGHEST_PROTOCOL)
-    pickled_md5 = md5_constructor(pickled + settings.SECRET_KEY).hexdigest()
-    return base64.encodestring(pickled + pickled_md5)
+    return base64.encodestring(pickled + get_query_hash(pickled))
 
 
 # Adapted from django.contrib.sessions.backends.base
@@ -412,7 +422,7 @@ def pickle_decode(session_data):
         session_data += u"="
     encoded_data = base64.decodestring(session_data)
     pickled, tamper_check = encoded_data[:-32], encoded_data[-32:]
-    pickled_md5 = md5_constructor(pickled + settings.SECRET_KEY).hexdigest()
+    pickled_md5 = get_query_hash(pickled)
     if pickled_md5 != tamper_check:
         raise SuspiciousOperation("User tampered with session cookie.")
     try:
@@ -421,3 +431,7 @@ def pickle_decode(session_data):
     # just return an empty dictionary (an empty session).
     except:
         return {}
+
+
+def get_query_hash(data):
+    return md5(data + settings.SECRET_KEY).hexdigest()
